window.Gleam.initSearch([{"doc":"gap","title":"gap","content":"# gap\n\n[![Package Version](https://img.shields.io/hexpm/v/gap)](https://hex.pm/packages/gap)\n[![Hex Docs](https://img.shields.io/badge/hex-docs-ffaff3)](https://hexdocs.pm/gap/)\n\nA Gleam library for comparing strings/lists and producing a textual (styled) representation of the differences.\n\nA typical styled output from the comparison can look like this:\n\n&lt;img src=&quot;https://github.com/JohnBjrk/gap/blob/main/static/example_diff_lucy.png?raw=true&quot; alt=&quot;Image of two strings with highlighted differences&quot; width=&quot;400vw&quot;&gt;\n\n## Installation\n\nIf available on Hex this package can be added to your Gleam project:\n\n```sh\ngleam add gap\n```\n\nDocumentation can be found at &lt;https://hexdocs.pm/gap&gt;.\n\n## Usage\n\n# Introduction\n\nGap implements string/list comparison by finding the longest common subsequence. The result of the comparison are two sequences \n(one for each of the compared strings/lists) consisting of subsequences that are annotated as matching or non-matching.\n\nFor example comparing the strings in the example above will look as follows:\n\n```gleam\nlet comparison =\ncompare_strings(\n    &quot;lucy in the sky with diamonds&quot;,\n    &quot;lucy is  the shy with diagrams&quot;,\n)\n|&gt; io.debug()\n// StringComparison(\n//   [\n//     Match([&quot;l&quot;, &quot;u&quot;, &quot;c&quot;, &quot;y&quot;, &quot; &quot;, &quot;i&quot;]),\n//     NoMatch([&quot;n&quot;]),\n//     Match([&quot; &quot;, &quot;t&quot;, &quot;h&quot;, &quot;e&quot;, &quot; &quot;, &quot;s&quot;]),\n//     NoMatch([&quot;k&quot;]),\n//     Match([&quot;y&quot;, &quot; &quot;, &quot;w&quot;, &quot;i&quot;, &quot;t&quot;, &quot;h&quot;, &quot; &quot;, &quot;d&quot;, &quot;i&quot;, &quot;a&quot;, &quot;m&quot;]),\n//     NoMatch([&quot;o&quot;, &quot;n&quot;, &quot;d&quot;]),\n//     Match([&quot;s&quot;]),\n//   ],\n//   [\n//     Match([&quot;l&quot;, &quot;u&quot;, &quot;c&quot;, &quot;y&quot;, &quot; &quot;, &quot;i&quot;]),\n//     NoMatch([&quot;s&quot;, &quot; &quot;]),\n//     Match([&quot; &quot;, &quot;t&quot;, &quot;h&quot;, &quot;e&quot;, &quot; &quot;, &quot;s&quot;]),\n//     NoMatch([&quot;h&quot;]),\n//     Match([&quot;y&quot;, &quot; &quot;, &quot;w&quot;, &quot;i&quot;, &quot;t&quot;, &quot;h&quot;, &quot; &quot;, &quot;d&quot;, &quot;i&quot;]),\n//     NoMatch([&quot;a&quot;, &quot;g&quot;, &quot;r&quot;]),\n//     Match([&quot;a&quot;, &quot;m&quot;, &quot;s&quot;]),\n//   ],\n// )\n```\n\n## Styling\n\nThis is useful information but a bit overwhelming to look at (specially for longer string) so the library\nhas some built in functions to display the differences using colors instead.\n\nUsing the same example again we can style the result and print it to the console\n\n```gleam\nlet comparison =\ncompare_strings(\n    &quot;lucy in the sky with diamonds&quot;,\n    &quot;lucy is  the shy with diagrams&quot;,\n)\n|&gt; to_styled()\nio.println(comparison.first)\nio.println(comparison.second)\n```\n\nThis will give us something similar to the output above.\n\n## Comparing list\n\nIt is also possible to compare lists with elements of arbitrary types. \n\n```gleam\npub type Warning {\n  Please\n  Mind\n  The(what: String)\n}\n\ncompare_lists([Mind, The(&quot;Gap&quot;)], [Please, Mind, The(&quot;What&quot;)])\n|&gt; io.debug()\n// ListComparison(\n//   [Match([Mind]), NoMatch([The(&quot;Gap&quot;)])],\n//   [NoMatch([Please]), Match([Mind]), NoMatch([The(&quot;What&quot;)])],\n// )\n```\n\n## Customize styling\n\nThe visual representation of the comparison can be customized. To do this use a `Styling` created from\nthe comparison that should be styled. This example uses [gleam_community/ansi](https://hexdocs.pm/gleam_community_ansi/index.html)\nto highlight the non-matches in different colors.\n\n```gleam\nlet comparison =\ncompare_strings(\n    &quot;Strings are made of smaller things&quot;,\n    &quot;Things are maybe smaller string&quot;,\n)\n|&gt; from_comparison()\n|&gt; highlight(\n    fn(first) { ansi.cyan(first) },\n    fn(second) { ansi.magenta(second) },\n    fn(matching) { matching },\n)\n|&gt; to_styled_comparison()\nio.println(comparison.first)\nio.println(comparison.second)\n```\n\nThis will output something similar to this\n\n&lt;img src=&quot;https://github.com/JohnBjrk/gap/blob/main/static/example_diff_things.png?raw=true&quot; alt=&quot;Image of two strings with highlighted differences&quot; width=&quot;400vw&quot;&gt;\n\n### Serialization\n\nFurthermore it is also possible to customize the styling by changing the way that the comparison is serialized. An easy way to do\nthis is to use the utility function `mk_generic_serializer` which creates a serializer which some specific separator and a hook\nfor surrounding the result with some content. Here is a somewhat contrived example\n\n```gleam\nlet comparison =\ncompare_lists([&quot;one&quot;, &quot;two&quot;, &quot;three&quot;], [&quot;two&quot;, &quot;two&quot;, &quot;tree&quot;])\n|&gt; from_comparison()\n|&gt; highlight(\n    fn(first) { first &lt;&gt; &quot; was not found in other&quot; },\n    fn(second) { second &lt;&gt; &quot; was not found in other&quot; },\n)\n|&gt; serialize(mk_generic_serializer(\n    &quot;, and &quot;,\n    fn(result) { &quot;Comparing the lists gave the following result: &quot; &lt;&gt; result },\n))\n|&gt; to_styled_comparison()\nio.println(comparison.first)\nio.println(comparison.second)\n// Comparing the lists gave the following result: &quot;one&quot; was not found in other, and &quot;two&quot; was found in other, and &quot;three&quot; was not found in other\n// Comparing the lists gave the following result: &quot;two&quot; was not found in other, and &quot;two&quot; was found in other, and &quot;tree&quot; was not found in other\n```\n\n### Custom serialization\n\nSerializers can of course have a custom implementation. The following example utilizes this together with custom highlighters,\ncreating a patch-like output (this is not exactly the same as patch-format since that shows the changes in relation to the original - to do \nthat both lists of matching/non-matching lines would need to be processed together)\n\n```gleam\nlet comparison =\ncompare_lists(\n    [\n    &quot;pub type Gap = List(EmptyString)&quot;, &quot;&quot;, &quot;pub type Traveler {&quot;,\n    &quot;  OnTrain&quot;, &quot;  OverGap(gap: Gap)&quot;, &quot;  OnPlatform&quot;, &quot;}&quot;,\n    ],\n    [\n    &quot;pub type Traveler {&quot;, &quot;  OnTrain&quot;, &quot;  OverGap(gap: String)&quot;,\n    &quot;  OnPlatform&quot;, &quot;}&quot;,\n    ],\n)\n|&gt; from_comparison()\n|&gt; highlight(\n    fn(first) { &quot;+&quot; &lt;&gt; first },\n    fn(second) { &quot;-&quot; &lt;&gt; second },\n    fn(matching) { &quot; &quot; &lt;&gt; matching },\n)\n|&gt; serialize(fn(part) {\n    case part {\n    Part(acc, lines, highlight) -&gt;\n        acc &lt;&gt; {\n        lines\n        |&gt; list.map(fn(line) { highlight(line) })\n        |&gt; string.join(&quot;\\n&quot;)\n        } &lt;&gt; &quot;\\n&quot;\n    All(result) -&gt; result\n    }\n})\n|&gt; to_styled_comparison()\nio.println(comparison.first)\nio.println(comparison.second)\n// +pub type Gap = List(EmptyString)\n// +\n//  pub type Traveler {\n//    OnTrain\n// +  OverGap(gap: Gap)\n//    OnPlatform\n//  }\n// \n//  pub type Traveler {\n//    OnTrain\n// -  OverGap(gap: String)\n//    OnPlatform\n//  }\n```\n","url":"index.html"},{"doc":"gap/comparison","title":"Comparison","content":"pub type Comparison(a) {\n  ListComparison(first: Segments(a), second: Segments(a))\n  StringComparison(\n    first: Segments(String),\n    second: Segments(String),\n  )\n}\n Comparison of two strings or lists\n\n The comparison consists of two lists of matched segments. The segments represent\n a sequence of succeeding matches or non-matches (up until the next match/non-match)\n \n For lists the elements in the segment will be same as the elements in the list, and\n for strings the elements will be the graphemes of the string\nListComparison(first: Segments(a), second: Segments(a))\n\n\nStringComparison(\n  first: Segments(String),\n  second: Segments(String),\n)\n\n\nSynonyms:\ngap/comparison.Comparison\ngap/comparison Comparison","url":"gap/comparison.html#Comparison"},{"doc":"gap/comparison","title":"Match","content":"pub type Match(a) {\n  Match(item: a)\n  NoMatch(item: a)\n}\n Indicating that the item has a matching (`Match`) or no matching (`NoMatch`) item in the\n other string/list\nMatch(item: a)\n\n\nNoMatch(item: a)\n\n\nSynonyms:\ngap/comparison.Match\ngap/comparison Match","url":"gap/comparison.html#Match"},{"doc":"gap/comparison","title":"Segments","content":"pub type Segments(a) =\n  List(Match(List(a)))\n List of segments of succeeding matches / non-matches\n\nSynonyms:\ngap/comparison.Segments\ngap/comparison Segments","url":"gap/comparison.html#Segments"},{"doc":"gap/comparison","title":"gap/comparison","content":"","url":"gap/comparison.html"},{"doc":"gap/styled_comparison","title":"StyledComparison","content":"pub type StyledComparison {\n  StyledComparison(first: String, second: String)\n}\n A comparison where the parts have been styled (serialized and highlighted)\nStyledComparison(first: String, second: String)\n\n\nSynonyms:\ngap/styled_comparison.StyledComparison\ngap/styled_comparison StyledComparison","url":"gap/styled_comparison.html#StyledComparison"},{"doc":"gap/styled_comparison","title":"gap/styled_comparison","content":"","url":"gap/styled_comparison.html"},{"doc":"gap/styling","title":"Highlighter","content":"pub type Highlighter =\n  fn(String) -&gt; String\n The `Highlighter`takes a string representation of the item that was not matching\n and should return a string representation that can be used to visually indicate that\n it is a non-matching item.\n\n The default implementation of the highlighters uses the \n [gleam_community/ansi](https://hexdocs.pm/gleam_community_ansi/index.html) library\n to set a different color for the item, but any type if indication can be used as long\n as it returns a valid string\n\nSynonyms:\ngap/styling.Highlighter\ngap/styling Highlighter","url":"gap/styling.html#Highlighter"},{"doc":"gap/styling","title":"Highlighters","content":"pub type Highlighters {\n  Highlighters(\n    first: Highlighter,\n    second: Highlighter,\n    matching: Highlighter,\n  )\n}\n Highlighters to use for indicating matches / non-matches\n\n `first` is used to highlight non-matches in the first string/list\n `second` is used to highlight non-matches in the second string/list\n `matching` is used to highlight matches in the both strings/lists\nHighlighters(\n  first: Highlighter,\n  second: Highlighter,\n  matching: Highlighter,\n)\n\n\nSynonyms:\ngap/styling.Highlighters\ngap/styling Highlighters","url":"gap/styling.html#Highlighters"},{"doc":"gap/styling","title":"Part","content":"pub type Part(a) {\n  Part(acc: String, part: List(a), highlight: Highlighter)\n  All(all: String)\n}\n `Part` is used to indicate to a custom serializer if it should produce a serialization\n based on a segment with items or the final string that contains already serialized segments\nPart(acc: String, part: List(a), highlight: Highlighter)\n `acc` the already serialized part of the result, `part` is the current segment that should be serialized and appended and `highlighter` is the `Highlighter` that can be used to indicate non-matching items\n\nAll(all: String)\n `all` is a string representing all serialized segments. This can be useful if some string should be prepended/appended to the final result\n\nSynonyms:\ngap/styling.Part\ngap/styling Part","url":"gap/styling.html#Part"},{"doc":"gap/styling","title":"Serializer","content":"pub type Serializer(a) =\n  fn(Part(a)) -&gt; String\n A `Serializer`can be used to create string representation of the comparison results\n\n See [serialize](#serialize) for adding custom serializers and [mk_generic_serializer](#mk_generic_serializer)\n\nSynonyms:\ngap/styling.Serializer\ngap/styling Serializer","url":"gap/styling.html#Serializer"},{"doc":"gap/styling","title":"Styling","content":"pub opaque type Styling(a)\n Styling of a `Comparison`\n\n See [from_comparison](#from_comparison)\n\nSynonyms:\ngap/styling.Styling\ngap/styling Styling","url":"gap/styling.html#Styling"},{"doc":"gap/styling","title":"first_highlight_default","content":"pub fn first_highlight_default(string: String) -&gt; String\n Default highlighter for the first string/list in the comparison\nSynonyms:\ngap/styling.first_highlight_default\ngap/styling first_highlight_default","url":"gap/styling.html#first_highlight_default"},{"doc":"gap/styling","title":"from_comparison","content":"pub fn from_comparison(comparison: Comparison(a)) -&gt; Styling(a)\n Create a new `Styling` from a `Comparison`\n\n The `Styling` can be customized by adding highlighters and a serializer\n See [highlight](#highlight) and [serialize](#serialize)\nSynonyms:\ngap/styling.from_comparison\ngap/styling from_comparison","url":"gap/styling.html#from_comparison"},{"doc":"gap/styling","title":"highlight","content":"pub fn highlight(\n  styling: Styling(a),\n  first: fn(String) -&gt; String,\n  second: fn(String) -&gt; String,\n  matching: fn(String) -&gt; String,\n) -&gt; Styling(a)\n Add highlighters to the `Styling`\n\n The highlighters are used to mark the matching/non-matching items in the\n first/second list/string\nSynonyms:\ngap/styling.highlight\ngap/styling highlight","url":"gap/styling.html#highlight"},{"doc":"gap/styling","title":"mk_generic_serializer","content":"pub fn mk_generic_serializer(\n  separator: String,\n  around: fn(String) -&gt; String,\n) -&gt; fn(Part(a)) -&gt; String\n Creates a generic serializer that uses `separator` between all items and calls\n `around` for possibility to prepend/append strings to the final result\nSynonyms:\ngap/styling.mk_generic_serializer\ngap/styling mk_generic_serializer","url":"gap/styling.html#mk_generic_serializer"},{"doc":"gap/styling","title":"no_highlight","content":"pub fn no_highlight(string: String) -&gt; String\n Default highlighter used for matching items\nSynonyms:\ngap/styling.no_highlight\ngap/styling no_highlight","url":"gap/styling.html#no_highlight"},{"doc":"gap/styling","title":"second_highlight_default","content":"pub fn second_highlight_default(string: String) -&gt; String\n Default highlighter for the second string/list in the comparison\nSynonyms:\ngap/styling.second_highlight_default\ngap/styling second_highlight_default","url":"gap/styling.html#second_highlight_default"},{"doc":"gap/styling","title":"serialize","content":"pub fn serialize(\n  styling: Styling(a),\n  serializer: fn(Part(a)) -&gt; String,\n) -&gt; Styling(a)\n Add a serializer to the `Styling`\n\n The serializer is used to create string representation of the items in the segments of the `Comparison`\n See [Part](#part) for details\n\n &gt; **NOTE:** `StringComparison` will always use the default string serializer (concatenating the graphemes).\n &gt; If there is a need for custom serialization of `StringComparison` convert the string to a list of \n &gt; graphemes and treat it as a `ListComparison`\nSynonyms:\ngap/styling.serialize\ngap/styling serialize","url":"gap/styling.html#serialize"},{"doc":"gap/styling","title":"to_styled_comparison","content":"pub fn to_styled_comparison(\n  styling: Styling(a),\n) -&gt; StyledComparison\n Creates a styled comparison using either custom highlighters/serializer if they where added or default\n highlighters and/or serializer\nSynonyms:\ngap/styling.to_styled_comparison\ngap/styling to_styled_comparison","url":"gap/styling.html#to_styled_comparison"},{"doc":"gap/styling","title":"gap/styling","content":"","url":"gap/styling.html"},{"doc":"gap/myers","title":"Edit","content":"pub type Edit(a) {\n  Eq(List(a))\n  Del(List(a))\n  Ins(List(a))\n}\n\nEq(List(a))\n\n\nDel(List(a))\n\n\nIns(List(a))\n\n\nSynonyms:\ngap/myers.Edit\ngap/myers Edit","url":"gap/myers.html#Edit"},{"doc":"gap/myers","title":"difference","content":"pub fn difference(\n  list1: List(a),\n  list2: List(a),\n) -&gt; List(Edit(a))\n The algorithm is outlined in the\n &quot;An O(ND) Difference Algorithm and Its Variations&quot; paper by E. Myers.\n \n Adapted from the implementation of &quot;myers_difference&quot; in Elixirs List module\nSynonyms:\ngap/myers.difference\ngap/myers difference","url":"gap/myers.html#difference"},{"doc":"gap/myers","title":"gap/myers","content":"","url":"gap/myers.html"},{"doc":"gap","title":"compare_lists","content":"pub fn compare_lists(\n  first_sequence: List(a),\n  second_sequence: List(a),\n) -&gt; Comparison(a)\n Compare two lists and return a `ListComparison` which will be styled as list\n when passed to `to_styled`\n \n Will use the default `myers` algorithm\nSynonyms:\ngap.compare_lists\ngap compare_lists","url":"gap.html#compare_lists"},{"doc":"gap","title":"compare_lists_with_algorithm","content":"pub fn compare_lists_with_algorithm(\n  first_sequence: List(a),\n  second_sequence: List(a),\n  algorithm: fn(List(a), List(a)) -&gt; Comparison(a),\n) -&gt; Comparison(a)\n Compare two lists and return a `ListComparison` which will be styled as list\n when passed to `to_styled`\n \n Algorithm can be used to select either `myers` or the legacy `lcs` algorithm\nSynonyms:\ngap.compare_lists_with_algorithm\ngap compare_lists_with_algorithm","url":"gap.html#compare_lists_with_algorithm"},{"doc":"gap","title":"compare_strings","content":"pub fn compare_strings(\n  first: String,\n  second: String,\n) -&gt; Comparison(String)\n Compare two string and return a `StringComparison` which will be styled as string\n when passed to `to_styled`\n \n Will use the default `myers` algorithm\nSynonyms:\ngap.compare_strings\ngap compare_strings","url":"gap.html#compare_strings"},{"doc":"gap","title":"compare_strings_with_algorithm","content":"pub fn compare_strings_with_algorithm(\n  first: String,\n  second: String,\n  algorithm: fn(List(String), List(String)) -&gt; Comparison(String),\n) -&gt; Comparison(String)\n Compare two string and return a `StringComparison` which will be styled as string\n when passed to `to_styled`\n \n Algorithm can be used to select either `myers` or the legacy `lcs` algorithm\nSynonyms:\ngap.compare_strings_with_algorithm\ngap compare_strings_with_algorithm","url":"gap.html#compare_strings_with_algorithm"},{"doc":"gap","title":"lcs","content":"pub fn lcs(\n  first_sequence: List(a),\n  second_sequence: List(a),\n) -&gt; Comparison(a)\n An adapter for the the `lcs` (longest common subsequence) algorithm.\n Intended to be use as an argument to `compare_strings_with_algorithm` or\n `compare_lists_with_algorithm`\nSynonyms:\ngap.lcs\ngap lcs","url":"gap.html#lcs"},{"doc":"gap","title":"myers","content":"pub fn myers(\n  first_sequence: List(a),\n  second_sequence: List(a),\n) -&gt; Comparison(a)\n An adapter for the the `myers` algorithm.\n Intended to be use as an argument to `compare_strings_with_algorithm` or\n `compare_lists_with_algorithm`\nSynonyms:\ngap.myers\ngap myers","url":"gap.html#myers"},{"doc":"gap","title":"to_styled","content":"pub fn to_styled(comparison: Comparison(a)) -&gt; StyledComparison\n Creates a `StyledComparison` from `Comparison` using default values for\n highting and serialization.\n\n ## Example\n\n \n &gt; compare_strings(&quot;abc&quot;, &quot;abe&quot;) |&gt; to_styled()\n \n This will return a `StyledComparison(first, second)` where &quot;c&quot; in `first` is green\n and &quot;e&quot; in `second` is red.\nSynonyms:\ngap.to_styled\ngap to_styled","url":"gap.html#to_styled"},{"doc":"gap","title":"gap","content":"","url":"gap.html"}]);